__author__ = u'Atamert \xd6l\xe7gen'
__copyright__ = u'Copyright 2009, \xd6l\xe7gen Bili\u015fim'
__credits__ = [u'Atamert \xd6l\xe7gen', 'Daniel Watkins']


__license__ = 'BSD'
__version__ = '1.1'
__maintainer__ = u'Atamert \xd6l\xe7gen'
__email__ = 'muhuk@muhuk.com'
__status__ = 'Production'
